@@ -1,14 +1,20 @@
 import re
-from ctypes import Structure, POINTER, pointer
+from ctypes import Structure, POINTER, pointer, cast
 from ctypes import (c_char, c_short, c_ushort, c_int, c_uint, c_long, c_ulong,
                     c_longlong, c_ulonglong, c_float, c_double, c_longdouble,
-                    c_bool)
+                    c_bool, c_size_t)
 import platform
 import numpy
 
 from railgun.cfuncs import cfdec_parse, choice_combinations, CJOINSTR
 from railgun.cdata import cddec_parse
-from railgun._helper import dict_override, strset
+from railgun.cmemsubsets import CMemSubSets
+from railgun._helper import (
+    dict_override, strset, subdict_by_prefix, subdict_by_filter)
+try:
+    from railgun import cstyle
+except ImportError:
+    pass
 
 """
 SimObject, its metaclass (MetaSimObject), and helper functions
@@ -24,6 +30,8 @@ DTYPE: numpy data type
 CTYPE: ctypes data types
     Fundamental data types of `ctypes` such as
     `c_int`, `c_double`, `c_float`, `c_bool`.
+VALTYPE : Value Type
+    {'scalar', 'array', 'object'}
 
 """
 
@@ -46,6 +54,7 @@ elif _architecture[0] == '64bit':
 else:
     raise RuntimeError(
         'Architecture %s is not supported' % str(_architecture))
+CDT2DTYPE['size_t'] = CDT2DTYPE['ulong']
 CDT2CTYPE = dict(char=c_char,
                  short=c_short, ushort=c_ushort,
                  int=c_int, uint=c_uint,
@@ -53,7 +62,7 @@ CDT2CTYPE = dict(char=c_char,
                  longlong=c_longlong, ulonglong=c_ulonglong,
                  float=c_float, double=c_double,
                  longdouble=c_longdouble,
-                 bool=c_bool,
+                 bool=c_bool, size_t=c_size_t,
                  )
 DTYPE2CDT = dict((numpy.dtype(v), k) for (k, v) in CDT2DTYPE.iteritems())
 DTYPE2CDT.update({
@@ -71,22 +80,38 @@ def POINTER_nth(ct, n):
         return POINTER_nth(POINTER(ct), n - 1)
 
 
-def as_ndim_pointer(arr, basetype, ndim):
-    if ndim == 1:
-        return arr.ctypes.data_as(POINTER(basetype))
+def as_ndim_pointer(arr, base_POINTER, ctpa, ctpas_remain):
+    if ctpas_remain:
+        next_ctpa = ctpas_remain[0]
+        next_ctpas_remain = ctpas_remain[1:]
+        prow = [as_ndim_pointer(row, base_POINTER, next_ctpa,
+                                next_ctpas_remain) for row in arr]
+        return ctpa(*prow)
     else:
-        ctp = POINTER_nth(basetype, ndim - 1)
-        ctpa = ctp * len(arr)
-        prow = [as_ndim_pointer(row, basetype, ndim - 1) for row in arr]
+        prow = [row.ctypes.data_as(base_POINTER) for row in arr]
         return ctpa(*prow)
 
 
 def ctype_getter(arr):
     basetype = CDT2CTYPE[DTYPE2CDT[arr.dtype]]
-    return as_ndim_pointer(arr, basetype, arr.ndim)
+    base_POINTER = POINTER(basetype)
+    if arr.ndim == 1:
+        return arr.ctypes.data_as(base_POINTER)
+    # make list of pointer type such as: int*, int**, int***, ...
+    _P = basetype
+    POINTER_list = []
+    for dummy in range(arr.ndim - 1):
+        _P = POINTER(_P)
+        POINTER_list.append(_P)
+    # callable to allocate N-dim "rows"
+    ctpa_list = []
+    for (i, num) in enumerate(arr.shape[:-1]):
+        ctpa_list.append(POINTER_list[- 1 - i] * num)
+    return as_ndim_pointer(arr, base_POINTER,
+                           ctpa_list[0], tuple(ctpa_list[1:]))
 
 
-def _gene_porp_scalar(key):
+def _gene_prop_scalar(key):
 
     def fget(self):
         return getattr(self._struct_, key)
@@ -96,13 +121,27 @@ def _gene_porp_scalar(key):
     return property(fget, fset)
 
 
-def _gene_porp_array(key):
+def _gene_prop_array(key):
 
     def fget(self):
-        return self._cdata_[key]
+        return self._cdatastore_[key]
 
     def fset(self, v):
-        self._cdata_[key][:] = v
+        self._cdatastore_[key][:] = v
+    return property(fget, fset)
+
+
+def _gene_prop_object(key):
+
+    def fget(self):
+        return self._cdatastore_[key]
+
+    def fset(self, v):
+        if not isinstance(v, self._cmems_parsed_[key].cdt):
+            raise ValueError('given value is not instance of %r' %
+                             self._cmems_parsed_[key].cdt)
+        self._cdatastore_[key] = v
+        setattr(self._struct_, key, v._cdata_)
     return property(fget, fset)
 
 
@@ -189,7 +228,7 @@ def get_cargs(self, defaults, kwds, keyorder):
     return [cargs_dict[k] for k in keyorder]
 
 
-def gene_cfpywrap(cfdec):
+def gene_cfpywrap(attrs, cfdec):
     """
     Generate python function given an object parsed by `cfuncs.cfdec_parse`
     """
@@ -212,6 +251,10 @@ def gene_cfpywrap(cfdec):
         # get cfunc
         cfname = cfdec.fnget(
             *get_cfunc_choices(defaults_choices, allkwds, choiceskeyorder))
+        if not self._cmemsubsets_parsed_.cfunc_is_callable(cfname):
+            raise ValueError(
+                'C function "%s" cannot be executed. Check flags %s.' %
+                (cfname, self._cmemsubsets_parsed_.getall()))
         cfunc = self._cfunc_loaded_[cfname]
         # get c-function arguments
         cargs = get_cargs(self, defaults_cargs, allkwds, cfkeyorder)
@@ -228,6 +271,10 @@ def gene_cfpywrap(cfdec):
             raise RuntimeError('c-function %s() terminates with code %d'
                                % (cfname, rcode))
     cfpywrap.func_name = cfdec.fname
+    # wrap it if there is wrap function
+    wrap_name = '_cwrap_%s' % cfdec.fname
+    if wrap_name in attrs:
+        cfpywrap = attrs[wrap_name](cfpywrap)
     return cfpywrap
 
 
@@ -285,7 +332,7 @@ def latest_attr(iter_of_obj, name, default=None):
     return default
 
 
-def attr_from_atttrs_or_bases(bases, attrs, name, default=None):
+def attr_from_attrs_or_bases(bases, attrs, name, default=None):
     """
     Get an attribute from `attrs` or from `bases` if not found in `attrs`
     """
@@ -315,19 +362,23 @@ def default_of_cmembers(cmems_parsed_list):
     """Get `cmems_default_{scalar, array}` from `cmems_parsed_list`"""
     cmems_default_scalar = dict(
         (parsed.vname, parsed.default) for parsed in cmems_parsed_list
-        if parsed.has_default and parsed.ndim == 0)
+        if parsed.has_default and parsed.valtype == 'scalar')
     cmems_default_array = dict(
         (parsed.vname, parsed.default) for parsed in cmems_parsed_list
-        if parsed.has_default and parsed.ndim > 0)
+        if parsed.has_default and parsed.valtype == 'array')
     return (cmems_default_scalar, cmems_default_array)
 
 
 def get_struct_class(cmems_parsed_list, cstructname):
-    fields = [(parsed.vname,
-               POINTER_nth(CDT2CTYPE[parsed.cdt], parsed.ndim))
-              for parsed in cmems_parsed_list]
-    # don't use `cmems_parsed.iteritems()` above or
-    # order of c-members will be lost!
+    fields = []
+    for parsed in cmems_parsed_list:
+        # don't use `cmems_parsed.iteritems()` above or
+        # order of c-members will be lost!
+        if parsed.valtype == 'object':
+            fields.append((parsed.vname, parsed.cdt._ctype_))
+        else:
+            ctype = POINTER_nth(CDT2CTYPE[parsed.cdt], parsed.ndim)
+            fields.append((parsed.vname, ctype))
 
     class StructClass(Structure):
         _fields_ = fields
@@ -337,7 +388,7 @@ def get_struct_class(cmems_parsed_list, cstructname):
 
 def array_alias_from_cmems_parsed(cmems_parsed):
     array_names = [vname for (vname, parsed) in cmems_parsed.iteritems()
-                   if parsed.ndim > 0]
+                   if parsed.valtype == 'array']
     array_alias = gene_array_alias(array_names)
     return staticmethod(array_alias)
 
@@ -365,18 +416,78 @@ def load_cfunc(cdll, cfuncs_parsed, struct_type_p, cfuncprefix, idxset):
 
 
 class MetaSimObject(type):
+    """
+    Meta-class for `SimObject`
+
+    Given Attributes
+    ----------------
+    _clibname_ : str
+        Name of your C shared library
+    _clibdir_ : str
+        Path of the C library directory
+    _cmembers_ : list of str
+        List of definition of C member
+    _cfuncs_ : list of str
+        List of definition of C function to be loaded
+    _cstructname_ : str, optional
+        Name of C struct (default is class name)
+    _cfuncprefix_ : str, optional
+        Prefix of C functions (default is ``_cstructname_ + '_'``)
+    _cwrap_{CFUNC_NAME} : function, optional
+        Wrapper for loaded C function `CFUNC_NAME`
+    _cmemsubsets_ : dict of dict of list, optional
+        Definition of subset of C members for controlling which
+        C members to be loaded. Example::
+
+            { 'name1': { 'cfuncs': ['f', 'g', 'h'], 'cmems': ['x', 'y'] },
+              'name2': { 'cfuncs': ['f', 'j', 'k'], 'cmems': ['y', 'z'] } }
+
+        Here, ``name1, name2`` are the name of C member subset,
+        ``f, g, h, j, k`` are the name of C functions, and
+        ``x, y, z`` are the name of C members.
+
+
+    Attributes to be set
+    --------------------
+    _cmems_parsed_ : dict
+        Key: name of C member; Value: object parsed by `cdata.cddec_parse`
+    _cmems_default_scalar_ : dict
+        Key: name of C member (scalar); Value: default value
+    _cmems_default_array_ : dict
+        Key: name of C member (array); Value: default value
+    _idxset_ : set
+        set of index
+    array_alias : function
+        Parser for "array alias" such as "a_1_2"
+    _struct_type_ : ctypes obejct
+        Subclass of `ctypes.Structure` generated from `_cmembers_`
+    _struct_type_p_ : ctypes obejct
+        Pointer type of _struct_type_
+    _cdll_ : ctypes object
+        Loaded C library
+    _cfunc_loaded_ : dict
+        Loaded C functions.
+        Key: Name of C function (without `_cfuncprefix_`);
+        Value: ctypes object
+    _cmemsubsets_parsed_ : CMemSubSets
+        An instance of CMemSubSets generated from `_cmemsubsets_`
+
+    Additionally, C member and C function will be added as attributes.
+
+    """
 
     def __new__(cls, clsname, bases, attrs):
-        clibdir = attr_from_atttrs_or_bases(bases, attrs, '_clibdir_')
-        clibname = attr_from_atttrs_or_bases(bases, attrs, '_clibname_')
-        cmembers = attr_from_atttrs_or_bases(bases, attrs, '_cmembers_')
-        cfuncs = attr_from_atttrs_or_bases(bases, attrs, '_cfuncs_')
-        if not all([clibdir, clibname, cmembers, cfuncs]):
+        clibdir = attr_from_attrs_or_bases(bases, attrs, '_clibdir_')
+        clibname = attr_from_attrs_or_bases(bases, attrs, '_clibname_')
+        cmembers = attr_from_attrs_or_bases(bases, attrs, '_cmembers_')
+        cfuncs = attr_from_attrs_or_bases(bases, attrs, '_cfuncs_')
+        if any(c is None for c in [clibdir, clibname, cmembers, cfuncs]):
             return type.__new__(cls, clsname, bases, attrs)
-        cstructname = attr_from_atttrs_or_bases(
+        cstructname = attr_from_attrs_or_bases(
             bases, attrs, '_cstructname_', clsname)
-        cfuncprefix = attr_from_atttrs_or_bases(
+        cfuncprefix = attr_from_attrs_or_bases(
             bases, attrs, '_cfuncprefix_', cstructname + CJOINSTR)
+        cmemsubsets = attr_from_attrs_or_bases(bases, attrs, '_cmemsubsets_')
 
         ## parse _cfuncs_
         cfuncs_parsed = parse_cfuncs(cfuncs)
@@ -400,52 +511,91 @@ class MetaSimObject(type):
 
         ## set getter/setter
         for (vname, parsed) in cmems_parsed.iteritems():
-            if parsed.ndim > 0:
-                attrs[vname] = _gene_porp_array(vname)
+            if parsed.valtype == 'array':
+                attrs[vname] = _gene_prop_array(vname)
+            elif parsed.valtype == 'scalar':
+                attrs[vname] = _gene_prop_scalar(vname)
+            elif parsed.valtype == 'object':
+                attrs[vname] = _gene_prop_object(vname)
             else:
-                attrs[vname] = _gene_porp_scalar(vname)
+                ValueError('valtype "%s" is not recognized' % parsed.valtype)
 
         ## load c-functions
         cdll = numpy.ctypeslib.load_library(clibname, clibdir)
         cfunc_loaded = load_cfunc(cdll, cfuncs_parsed, struct_type_p,
                                   cfuncprefix, idxset)
-        attrs.update(_cdll_=cdll, _cfunc_loaded_=cfunc_loaded)
+        attrs.update(
+            _cdll_=cdll,
+            _cfunc_loaded_=cfunc_loaded,
+            _cmemsubsets_parsed_=CMemSubSets(
+                cmemsubsets, set(cfunc_loaded), set(cmems_parsed)),
+            )
         for (fname, parsed) in cfuncs_parsed.iteritems():
-            attrs[fname] = gene_cfpywrap(parsed)
+            attrs[fname] = gene_cfpywrap(attrs, parsed)
 
         return type.__new__(cls, clsname, bases, attrs)
 
 
 class SimObject(object):
     __metaclass__ = MetaSimObject
+    _calloc_ = True  # if True, use cstyle.CStyle to allocate memory
 
     def __init__(self, **kwds):
+        """
+        Allocate C members to construct `SimObject`
+
+        Parameters
+        ----------
+        _calloc_ : bool, optional
+            If True, use cstyle.CStyle to allocate memory (default: True).
+        _cmemsubsets_{FLAG_NAME} : bool, optional
+            Set flag of `FLAG_NAME` which is defined by `_cmemsubsets_`
+            attribute.
+
+        Other keyword arguments will be name of C member or
+        "array alias" of array C member.
+
+        """
+        if '_calloc_' in kwds:
+            self._calloc_ = kwds['_calloc_']
+            del kwds['_calloc_']
+
+        # copy is needed otherwise self._cmemsubsets_parsed_ is shared by
+        # all instance of SimObject
+        self._cmemsubsets_parsed_ = self._cmemsubsets_parsed_.copy()
+        cmemsubsets_kwds = subdict_by_prefix(kwds, '_cmemsubsets_',
+                                             remove_original=True)
+        self._cmemsubsets_parsed_.set(**cmemsubsets_kwds)
+
         self._set_all(**kwds)
 
+    def _is_cmem_scalar(self, name):
+        return (name in self._cmems_parsed_ and
+                self._cmems_parsed_[name].valtype == 'scalar')
+
+    def _is_cmem_array(self, name):
+        return (name in self._cmems_parsed_ and
+                self._cmems_parsed_[name].valtype == 'array')
+
+    def _is_cmem_object(self, name):
+        return (name in self._cmems_parsed_ and
+                self._cmems_parsed_[name].valtype == 'object')
+
     def _set_all(self, **kwds):
-        # searching invalid keyword arguments
-        non_cmem_keys = set(kwds) - set(self._cmems_parsed_)
-        cmem_keys = set(kwds) - non_cmem_keys
-        cmem_keys_scalar = set(
-            [k for k in cmem_keys if self._cmems_parsed_[k].ndim == 0])
-        cmem_keys_array = set(
-            [k for k in cmem_keys if self._cmems_parsed_[k].ndim > 0])
-        undefined_keywords = set()
-        for key in non_cmem_keys:
-            if not self.array_alias(key):
-                undefined_keywords.add(key)
-        if undefined_keywords:
+        # decompose keyword arguments into its disjoint subsets
+        kwds_scalar = subdict_by_filter(kwds, self._is_cmem_scalar, True)
+        kwds_array = subdict_by_filter(kwds, self._is_cmem_array, True)
+        kwds_array_alias = subdict_by_filter(kwds, self.array_alias, True)
+        kwds_object = subdict_by_filter(kwds, self._is_cmem_object, True)
+        if kwds:  # there should not be remaining arguments
             raise ValueError(
-                "undefined keyword arguments: %s" % strset(undefined_keywords))
-        kwds_cmem_scalar = dict([(k, kwds[k]) for k in cmem_keys_scalar])
-        kwds_cmem_array = dict([(k, kwds[k]) for k in cmem_keys_array])
-        kwds_non_cmem = dict([(k, kwds[k]) for k in non_cmem_keys])
+                "undefined keyword arguments: %s" % kwds)
         # allocate struct
         self._struct_ = self._struct_type_()
         self._struct_p_ = pointer(self._struct_)
         # set scalar variables including num_*
         scalarvals = dict_override(
-            self._cmems_default_scalar_, kwds_cmem_scalar, addkeys=True)
+            self._cmems_default_scalar_, kwds_scalar, addkeys=True)
         numkeyset = set('num_%s' % i for i in self._idxset_)
         num_lack = numkeyset - set(scalarvals)
         if num_lack:
@@ -453,9 +603,15 @@ class SimObject(object):
         self.setv(**scalarvals)
         # allocate C array data and set the defaults
         self._set_cdata()
+        cmems_default_array = self._cmems_default_array_
+        array_allocated = filter(  # remove if not allocated
+            self._cmemsubsets_parsed_.cmem_need_alloc, cmems_default_array)
+        cmems_default_array_allocated = dict((k, cmems_default_array[k])
+                                             for k in array_allocated)
         self.setv(**dict_override(
-            self._cmems_default_array_, kwds_cmem_array, addkeys=True))
-        self.setv(**kwds_non_cmem)
+            cmems_default_array_allocated, kwds_array, addkeys=True))
+        self.setv(**kwds_array_alias)
+        self.setv(**kwds_object)
 
     def setv(self, **kwds):
         """
@@ -469,7 +625,7 @@ class SimObject(object):
             alias = self.array_alias(key)
             if alias:
                 (name, index) = alias
-                self._cdata_[name][index] = val
+                self._cdatastore_[name][index] = val
             else:
                 setattr(self, key, val)
 
@@ -498,7 +654,9 @@ class SimObject(object):
             return mems
 
     def num(self, *args):
-        """Get size of array (num_'i') along given index ('i') """
+        """
+        Get shape of array (Ni, Nj, ...) along given index ('i', 'j', ...)
+        """
         if len(args) == 1:
             args = [a.strip() for a in args[0].split(',')]
         if set(args) > self._idxset_:
@@ -511,14 +669,25 @@ class SimObject(object):
             return nums
 
     def _set_cdata(self):
-        self._cdata_ = {}  # keep memory for arrays
+        self._cdatastore_ = {}  # keep memory for arrays
+        cmem_need_alloc = self._cmemsubsets_parsed_.cmem_need_alloc
         for (vname, parsed) in self._cmems_parsed_.iteritems():
-            if parsed.ndim > 0:
-                shape = tuple(getattr(self, 'num_%s' % i) for i in parsed.idx)
+            if parsed.valtype == 'array' and cmem_need_alloc(vname):
+                shape = tuple(
+                    int(i) if i.isdigit() else getattr(self, 'num_%s' % i)
+                    for i in parsed.idx)
                 dtype = CDT2DTYPE[parsed.cdt]
                 arr = numpy.zeros(shape, dtype=dtype)
-                self._cdata_[vname] = arr
-                setattr(self._struct_, vname, ctype_getter(arr))
+                self._cdatastore_[vname] = arr
+                if self._calloc_ and 1 < arr.ndim <= cstyle.MAXDIM:
+                    cs = cstyle.CStyle(arr)
+                    self._cdatastore_['CStyle:%s' % vname] = cs
+                    ptr = cast(
+                        cs.pointer,
+                        POINTER_nth(CDT2CTYPE[parsed.cdt], parsed.ndim))
+                else:
+                    ptr = ctype_getter(arr)
+                setattr(self._struct_, vname, ptr)
 
     def _check_index_in_range(self, arg_val_list):
         """
